package com.kuyuntech.vrv.platform.configuration.core;

import com.alibaba.fastjson.JSONObject;
import com.kuyuntech.vrv.coreapi.bean.core.DeviceAlarmConfigBean;
import com.kuyuntech.vrv.coreapi.bean.core.DeviceAlarmRecordBean;
import com.kuyuntech.vrv.coreapi.bean.core.DeviceParamBean;
import com.kuyuntech.vrv.coreapi.bean.mqtt.DeviceParam;
import com.kuyuntech.vrv.coreapi.service.core.DeviceAlarmConfigService;
import com.kuyuntech.vrv.coreapi.service.core.DeviceAlarmRecordService;
import com.kuyuntech.vrv.coreapi.service.core.DeviceParamService;
import org.apache.commons.lang3.StringUtils;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.integration.annotation.ServiceActivator;
import org.springframework.integration.channel.DirectChannel;
import org.springframework.integration.core.MessageProducer;
import org.springframework.integration.mqtt.core.DefaultMqttPahoClientFactory;
import org.springframework.integration.mqtt.core.MqttPahoClientFactory;
import org.springframework.integration.mqtt.inbound.MqttPahoMessageDrivenChannelAdapter;
import org.springframework.integration.mqtt.outbound.MqttPahoMessageHandler;
import org.springframework.integration.mqtt.support.DefaultPahoMessageConverter;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageHandler;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Configuration
public class MqttConfiguration {


    @Autowired
    private DeviceParamService deviceParamService;
    @Autowired
    DeviceAlarmRecordService deviceAlarmRecordService;
    @Autowired
    DeviceAlarmConfigService deviceAlarmConfigService;


    // 消费消息

    /**
     * 创建MqttPahoClientFactory，设置MQTT Broker连接属性，如果使用SSL验证，也在这里设置。
     *
     * @return factory
     */
    @Bean
    public MqttPahoClientFactory mqttClientFactory() {
        DefaultMqttPahoClientFactory factory = new DefaultMqttPahoClientFactory();
        MqttConnectOptions options = new MqttConnectOptions();

        // 设置代理端的URL地址，可以是多个
        options.setServerURIs(new String[]{"tcp://47.107.255.78:1883"});

        factory.setConnectionOptions(options);
        return factory;
    }

    /**
     * 入站通道
     */
    @Bean
    public MessageChannel mqttInputChannel() {
        return new DirectChannel();
    }

    /**
     * 入站
     */
    @Bean
    public MessageProducer inbound() {
        // Paho客户端消息驱动通道适配器，主要用来订阅主题
        MqttPahoMessageDrivenChannelAdapter adapter = new MqttPahoMessageDrivenChannelAdapter("A-paho",
                mqttClientFactory(), "mqtt/335102012203A500/data", "mqtt/335102012203A500/cmd", "battery", "+/sensor");
        adapter.setCompletionTimeout(5000);

        // Paho消息转换器
        DefaultPahoMessageConverter defaultPahoMessageConverter = new DefaultPahoMessageConverter();
        // 按字节接收消息
//        defaultPahoMessageConverter.setPayloadAsBytes(true);
        adapter.setConverter(defaultPahoMessageConverter);
        adapter.setQos(1); // 设置QoS
        adapter.setOutputChannel(mqttInputChannel());
        return adapter;
    }

    @Bean
    // ServiceActivator注解表明：当前方法用于处理MQTT消息，inputChannel参数指定了用于消费消息的channel。
    @ServiceActivator(inputChannel = "mqttInputChannel")
    public MessageHandler handler() {
        return message -> {

            //接受到的消息
            String payload = message.getPayload().toString();

            // byte[] bytes = (byte[]) message.getPayload(); // 收到的消息是字节格式
            String topic = message.getHeaders().get("mqtt_receivedTopic").toString();

            // 根据主题分别进行消息处理。
            if (topic.matches(".+/sensor")) { // 匹配：1/sensor
                String sensorSn = topic.split("/")[0];
                System.out.println("传感器" + sensorSn + ": 的消息： " + payload);
            } else if (topic.equals("collector")) {
                System.out.println("采集器的消息：" + payload);
            } else {
                //设备传入
                System.out.println("丢弃消息：主题[" + topic + "]，负载：" + payload);

                //String转Json
                JSONObject jsonObject = JSONObject.parseObject(payload);

                String devices = jsonObject.getString("devices");

                //取里面
                List<DeviceParam> list = JSONObject.parseArray(devices, DeviceParam.class);

                //判断是否单个
                if (list.size() == 1) {
                    System.out.println("这是单个数据的处理");

                    //取deviceData
                    HashMap<String, String> deviceData = list.get(0).getDeviceData();
                    System.out.println(deviceData + "=======");

                    //判断传来的是不是报警
                    for (Map.Entry<String, String> entry : deviceData.entrySet()) {

                        String name = entry.getKey();
                        String value = entry.getValue();

                        System.out.println(name == "WD_GBJ");
                        int var = Integer.parseInt(value);

                        if ((name == "WD_GBJ" && var == 1)
                                || (name == "WD_DBJ" && var == 1)
                                || (name == "SD_GBJ" && var == 1)
                                || (name == "SD_DBJ" && var == 1)
                                || (name == "SFJ_GZ" && var == 1)
                                || (name == "PFJ_GZ" && var == 1)
                                || (name == "FLZJ_GZ" && var == 1)
                                || (name == "GW_BJ" && var == 1)
                                || (name == "LWDS_CX" && var == 1)
                                || (name == "LWDS_ZX" && var == 1)
                                || (name == "JZ_GZ" && var == 1)
                                || (name == "QFYC_BJ" && var == 1)
                                || (name == "JZ_JT" && var == 1)) {
                            System.out.println("这是一个报警！！");
                            //找到对应的报警设置
                            DeviceAlarmConfigBean deviceAlarmConfigBean = deviceAlarmConfigService.findByName(name);
                            //添加一个报警记录
                            DeviceAlarmRecordBean deviceAlarmRecordBean = new DeviceAlarmRecordBean();
                            deviceAlarmRecordBean.setAlarmName(deviceAlarmConfigBean.getAlarmName());
                            deviceAlarmRecordBean.setAlarmType(deviceAlarmConfigBean.getAlarmType());
                            deviceAlarmRecordBean.setSn("CK1");
                            deviceAlarmRecordBean.setContent(deviceAlarmConfigBean.getAlarmName());
                            deviceAlarmRecordBean.setAlarmConfigCode(deviceAlarmConfigBean.getCode());
                            deviceAlarmRecordBean.setStartTime(new Date());
                            //状态：1表示未处理，0表示已处理
                            deviceAlarmRecordBean.setStatus(value);
                            deviceAlarmRecordService.add(deviceAlarmRecordBean);
                        }

                        if ((name == "WD_GBJ" && var == 0)
                                || (name == "WD_DBJ" && var == 0)
                                || (name == "SD_GBJ" && var == 0)
                                || (name == "SD_DBJ" && var == 0)
                                || (name == "SFJ_GZ" && var == 0)
                                || (name == "PFJ_GZ" && var == 0)
                                || (name == "FLZJ_GZ" && var == 0)
                                || (name == "GW_BJ" && var == 0)
                                || (name == "LWDS_CX" && var == 0)
                                || (name == "LWDS_ZX" && var == 0)
                                || (name == "JZ_GZ" && var == 0)
                                || (name == "QFYC_BJ" && var == 0)
                                || (name == "JZ_JT" && var == 0)) {
                            System.out.println("这是一个报警处理，更新为已处理和结束时间！！");
                            //找到对应的报警设置
                            DeviceAlarmConfigBean deviceAlarmConfigBean = deviceAlarmConfigService.findByName(name);

                            //状态：1表示未处理，0表示已处理
                            deviceAlarmRecordService.updateParam(deviceAlarmConfigBean.getCode());
                        }


                    }


                } else {
                    System.out.println("这是所有数据的处理");

                    //CK1
                    String deviceId = list.get(0).getDeviceId();
                    //CK2
                    String deviceId2 = list.get(1).getDeviceId();

                    //取deviceData
                    HashMap<String, String> deviceData = list.get(0).getDeviceData();
                    //CK2的deviceData
                    HashMap<String, String> deviceData2 = list.get(1).getDeviceData();

                    long id = 199;

                    //更新数据库
                    for (Map.Entry<String, String> entry : deviceData.entrySet()) {
                        DeviceParamBean deviceParamBean = new DeviceParamBean();

                        deviceParamBean.setSn(deviceId);

                        String name = entry.getKey();
                        String value = entry.getValue();


                        deviceParamBean.setId(id);

                        id = id + 1;

                        deviceParamBean.setName(name);
                        deviceParamBean.setValue(value);

                        deviceParamBean = this.deviceParamService.updateParam(deviceParamBean);

                        System.out.println("id为：" + id + "-- 更新后的数据：" + deviceParamBean);

                    }

                    //更新数据库
                    for (Map.Entry<String, String> entry : deviceData2.entrySet()) {
                        DeviceParamBean deviceParamBean = new DeviceParamBean();
                        deviceParamBean.setSn(deviceId2);

                        for (long i = 308; i < 310; i++) {
                            deviceParamBean.setId(i);
                        }

                        String name = entry.getKey();
                        String value = entry.getValue();

                        deviceParamBean.setName(name);
                        deviceParamBean.setValue(value);

                        deviceParamBean = this.deviceParamService.updateParam(deviceParamBean);
                    }
                }
            }

        };
    }

    // 发送消息

    /**
     * 出站通道
     */
    @Bean
    public MessageChannel mqttOutboundChannel() {
        return new DirectChannel();
    }

    /**
     * 出站
     */
    @Bean
    @ServiceActivator(inputChannel = "mqttOutboundChannel")
    public MessageHandler outbound() {

        // 发送消息和消费消息Channel可以使用相同MqttPahoClientFactory
        MqttPahoMessageHandler messageHandler = new MqttPahoMessageHandler("publishClient", mqttClientFactory());
        messageHandler.setAsync(true); // 如果设置成true，即异步，发送消息时将不会阻塞。
        messageHandler.setDefaultTopic("command");
        messageHandler.setDefaultQos(1); // 设置默认QoS

        // Paho消息转换器
        DefaultPahoMessageConverter defaultPahoMessageConverter = new DefaultPahoMessageConverter();

        // defaultPahoMessageConverter.setPayloadAsBytes(true); // 发送默认按字节类型发送消息
        messageHandler.setConverter(defaultPahoMessageConverter);
        return messageHandler;
    }

}
